@@ -3,6 +3,8 @@
 #   Code output module
 #
 
+from __future__ import absolute_import
+
 import cython
 cython.declare(os=object, re=object, operator=object,
                Naming=object, Options=object, StringEncoding=object,
@@ -21,13 +23,13 @@ try:
 except ImportError:
     import md5 as hashlib
 
-import Naming
-import Options
-import StringEncoding
-from Cython import Utils
-from Scanning import SourceDescriptor
-from Cython.StringIOTree import StringIOTree
-import DebugFlags
+from . import Naming
+from . import Options
+from . import DebugFlags
+from . import StringEncoding
+from .. import Utils
+from .Scanning import SourceDescriptor
+from ..StringIOTree import StringIOTree
 
 try:
     from __builtin__ import basestring
@@ -44,19 +46,19 @@ non_portable_builtins_map = {
     'basestring'    : ('PY_MAJOR_VERSION >= 3', 'str'),
     'xrange'        : ('PY_MAJOR_VERSION >= 3', 'range'),
     'raw_input'     : ('PY_MAJOR_VERSION >= 3', 'input'),
-    'BaseException' : ('PY_VERSION_HEX < 0x02050000', 'Exception'),
     }
 
 basicsize_builtins_map = {
     # builtins whose type has a different tp_basicsize than sizeof(...)
-    'PyTypeObject' : 'PyHeapTypeObject',
-    }
+    'PyTypeObject': 'PyHeapTypeObject',
+}
 
 uncachable_builtins = [
     # builtin names that cannot be cached because they may or may not
     # be available at import time
     'WindowsError',
-    ]
+    '_',  # e.g. gettext
+]
 
 modifier_output_mapper = {
     'inline': 'CYTHON_INLINE'
@@ -152,7 +154,7 @@ class UtilityCodeBase(object):
             replace_comments = re.compile(r'^\s*#.*').sub
         else:
             comment = '/'
-            replace_comments = re.compile(r'^\s*//.*|^\s*/\*[^*]*\*/').sub
+            replace_comments = re.compile(r'^\s*//.*|/\*[^*]*\*/').sub
         match_special = re.compile(
             (r'^%(C)s{5,30}\s*(?P<name>(?:\w|\.)+)\s*%(C)s{5,30}|'
              r'^%(C)s+@(?P<tag>\w+)\s*:\s*(?P<value>(?:\w|[.:])+)'
@@ -249,7 +251,13 @@ class UtilityCodeBase(object):
                     continue
                 # only pass lists when we have to: most argument expect one value or None
                 if name == 'requires':
-                    values = [ cls.load(dep, from_file, **orig_kwargs) for dep in values ]
+                    if orig_kwargs:
+                        values = [cls.load(dep, from_file, **orig_kwargs)
+                                  for dep in sorted(values)]
+                    else:
+                        # dependencies are rarely unique, so use load_cached() when we can
+                        values = [cls.load_cached(dep, from_file)
+                                  for dep in sorted(values)]
                 elif not values:
                     values = None
                 elif len(values) == 1:
@@ -269,16 +277,16 @@ class UtilityCodeBase(object):
         return cls(**kwargs)
 
     @classmethod
-    def load_cached(cls, utility_code_name, from_file=None, _cache={}):
+    def load_cached(cls, utility_code_name, from_file=None, __cache={}):
         """
         Calls .load(), but using a per-type cache based on utility name and file name.
         """
         key = (cls, from_file, utility_code_name)
         try:
-            return _cache[key]
+            return __cache[key]
         except KeyError:
             pass
-        code = _cache[key] = cls.load(utility_code_name, from_file)
+        code = __cache[key] = cls.load(utility_code_name, from_file)
         return code
 
     @classmethod
@@ -362,7 +370,7 @@ class UtilityCode(UtilityCodeBase):
     def specialize(self, pyrex_type=None, **data):
         # Dicts aren't hashable...
         if pyrex_type is not None:
-            data['type'] = pyrex_type.declaration_code('')
+            data['type'] = pyrex_type.empty_declaration_code()
             data['type_name'] = pyrex_type.specialization_name()
         key = tuple(sorted(data.items()))
         try:
@@ -446,7 +454,7 @@ def sub_tempita(s, context, file=None, name=None):
     elif name:
         context['__name'] = name
 
-    from Cython.Tempita import sub
+    from ..Tempita import sub
     return sub(s, **context)
 
 class TempitaUtilityCode(UtilityCode):
@@ -618,7 +626,7 @@ class FunctionState(object):
 
         A C string referring to the variable is returned.
         """
-        if type.is_const:
+        if type.is_const and not type.is_reference:
             type = type.const_base_type
         if not type.is_pyobject and not type.is_memoryviewslice:
             # Make manage_ref canonical, so that manage_ref will always mean
@@ -636,7 +644,7 @@ class FunctionState(object):
             self.temps_allocated.append((result, type, manage_ref, static))
         self.temps_used_type[result] = (type, manage_ref)
         if DebugFlags.debug_temp_code_comments:
-            self.owner.putln("/* %s allocated */" % result)
+            self.owner.putln("/* %s allocated (%s) */" % (result, type))
 
         if self.collect_temps_stack:
             self.collect_temps_stack[-1].add((result, type))
@@ -959,14 +967,15 @@ class GlobalState(object):
             w.putln("")
             w.putln("static void __Pyx_CleanupGlobals(void) {")
 
-        #
-        # utility_code_def
-        #
+        code = self.parts['utility_code_proto']
+        code.putln("")
+        code.putln("/* --- Runtime support code (head) --- */")
+
         code = self.parts['utility_code_def']
         if self.emit_linenums:
             code.write('\n#line 1 "cython_utility"\n')
         code.putln("")
-        code.putln("/* Runtime support code */")
+        code.putln("/* --- Runtime support code --- */")
 
     def finalize_main_c_code(self):
         self.close_global_decls()
@@ -1309,13 +1318,13 @@ class GlobalState(object):
     # File name state
     #
 
-    def lookup_filename(self, filename):
+    def lookup_filename(self, source_desc):
         try:
-            index = self.filename_table[filename]
+            index = self.filename_table[source_desc.get_filenametable_entry()]
         except KeyError:
             index = len(self.filename_list)
-            self.filename_list.append(filename)
-            self.filename_table[filename] = index
+            self.filename_list.append(source_desc)
+            self.filename_table[source_desc.get_filenametable_entry()] = index
         return index
 
     def commented_file_contents(self, source_desc):
@@ -1615,11 +1624,11 @@ class CCodeWriter(object):
             self.level += 1
 
     def putln_tempita(self, code, **context):
-        from Cython.Tempita import sub
+        from ..Tempita import sub
         self.putln(sub(code, **context))
 
     def put_tempita(self, code, **context):
-        from Cython.Tempita import sub
+        from ..Tempita import sub
         self.put(sub(code, **context))
 
     def increase_indent(self):
@@ -1695,7 +1704,7 @@ class CCodeWriter(object):
             if type.is_pyobject:
                 self.putln("%s = NULL;" % decl)
             elif type.is_memoryviewslice:
-                import MemoryView
+                from . import MemoryView
                 self.putln("%s = %s;" % (decl, MemoryView.memslice_entry_init))
             else:
                 self.putln("%s%s;" % (static and "static " or "", decl))
@@ -1736,7 +1745,7 @@ class CCodeWriter(object):
             return entry.cname
 
     def as_pyobject(self, cname, type):
-        from PyrexTypes import py_object_type, typecast
+        from .PyrexTypes import py_object_type, typecast
         return typecast(py_object_type, type, cname)
 
     def put_gotref(self, cname):
@@ -1864,12 +1873,12 @@ class CCodeWriter(object):
             self.put_var_xdecref_clear(entry)
 
     def put_incref_memoryviewslice(self, slice_cname, have_gil=False):
-        import MemoryView
+        from . import MemoryView
         self.globalstate.use_utility_code(MemoryView.memviewslice_init_code)
         self.putln("__PYX_INC_MEMVIEW(&%s, %d);" % (slice_cname, int(have_gil)))
 
     def put_xdecref_memoryviewslice(self, slice_cname, have_gil=False):
-        import MemoryView
+        from . import MemoryView
         self.globalstate.use_utility_code(MemoryView.memviewslice_init_code)
         self.putln("__PYX_XDEC_MEMVIEW(&%s, %d);" % (slice_cname, int(have_gil)))
 
@@ -1877,7 +1886,7 @@ class CCodeWriter(object):
         self.put_xgiveref("%s.memview" % slice_cname)
 
     def put_init_to_py_none(self, cname, type, nanny=True):
-        from PyrexTypes import py_object_type, typecast
+        from .PyrexTypes import py_object_type, typecast
         py_none = typecast(type, py_object_type, "Py_None")
         if nanny:
             self.putln("%s = %s; __Pyx_INCREF(Py_None);" % (cname, py_none))
@@ -1902,7 +1911,7 @@ class CCodeWriter(object):
                 # that's better than ours.
                 elif allow_skip:
                     return
-        from TypeSlots import method_coexist
+        from .TypeSlots import method_coexist
         if entry.doc:
             doc_code = entry.doc_cname
         else:
@@ -1912,7 +1921,7 @@ class CCodeWriter(object):
             if entry.is_special:
                 method_flags += [method_coexist]
             self.putln(
-                '{__Pyx_NAMESTR("%s"), (PyCFunction)%s, %s, __Pyx_DOCSTR(%s)}%s' % (
+                '{"%s", (PyCFunction)%s, %s, %s}%s' % (
                     entry.name,
                     entry.func_cname,
                     "|".join(method_flags),
@@ -1980,7 +1989,7 @@ class CCodeWriter(object):
         return self.putln("if (%s < 0) %s" % (value, self.error_goto(pos)))
 
     def put_error_if_unbound(self, pos, entry, in_nogil_context=False):
-        import ExprNodes
+        from . import ExprNodes
         if entry.from_closure:
             func = '__Pyx_RaiseClosureNameError'
             self.globalstate.use_utility_code(
@@ -2066,7 +2075,7 @@ class CCodeWriter(object):
         self.funcstate.uses_error_indicator = True
         self.putln('__Pyx_AddTraceback("%s", %s, %s, %s);' % format_tuple)
 
-    def put_unraisable(self, qualified_name):
+    def put_unraisable(self, qualified_name, nogil=False):
         """
         Generate code to print a Python warning for an unraisable exception.
 
@@ -2077,15 +2086,16 @@ class CCodeWriter(object):
             Naming.clineno_cname,
             Naming.lineno_cname,
             Naming.filename_cname,
-            int(self.globalstate.directives['unraisable_tracebacks'])
+            self.globalstate.directives['unraisable_tracebacks'],
+            nogil,
         )
         self.funcstate.uses_error_indicator = True
-        self.putln('__Pyx_WriteUnraisable("%s", %s, %s, %s, %s);' % format_tuple)
+        self.putln('__Pyx_WriteUnraisable("%s", %s, %s, %s, %d, %d);' % format_tuple)
         self.globalstate.use_utility_code(
             UtilityCode.load_cached("WriteUnraisableException", "Exceptions.c"))
 
-    def put_trace_declarations(self):
-        self.putln('__Pyx_TraceDeclarations')
+    def put_trace_declarations(self, codeobj=None):
+        self.putln('__Pyx_TraceDeclarations(%s)' % (codeobj or 'NULL'))
 
     def put_trace_call(self, name, pos):
         self.putln('__Pyx_TraceCall("%s", %s[%s], %s);' % (name, Naming.filetable_cname, self.lookup_filename(pos[0]), pos[1]))
