@@ -2,6 +2,8 @@
 #  Cython - Compilation-wide options and pragma declarations
 #
 
+from __future__ import absolute_import
+
 # Perform lookups on builtin names only once, at module initialisation
 # time.  This will prevent the module from getting imported if a
 # builtin name that it uses cannot be found during initialisation.
@@ -34,6 +36,12 @@ warning_errors = False
 # you should disable this option and also 'cache_builtins'.
 error_on_unknown_names = True
 
+# Make uninitialized local variable reference a compile time error.
+# Python raises UnboundLocalError at runtime, whereas this option makes
+# them a compile time error. Note that this option affects only variables
+# of "python object" type.
+error_on_uninitialized = True
+
 # This will convert statements of the form "for i in range(...)"
 # to "for i from ..." when i is a cdef'd integer type, and the direction
 # (i.e. sign of step) can be determined.
@@ -55,10 +63,6 @@ lookup_module_cpdef = False
 # executes the body of this module.
 embed = None
 
-# Disables function redefinition, allowing all functions to be declared at
-# module creation time. For legacy code only, needed for some circular imports.
-disable_function_redefinition = False
-
 # In previous iterations of Cython, globals() gave the first non-Cython module
 # globals in the call stack.  Sage relies on this behavior for variable injection.
 old_style_globals = False
@@ -66,11 +70,17 @@ old_style_globals = False
 # Allows cimporting from a pyx file without a pxd file.
 cimport_from_pyx = False
 
-
 # max # of dims for buffers -- set lower than number of dimensions in numpy, as
 # slices are passed by value and involve a lot of copying
 buffer_max_dims = 8
 
+# Number of function closure instances to keep in a freelist (0: no freelists)
+closure_freelist_size = 8
+
+# Should tp_clear() set object fields to None instead of clearing them to NULL?
+clear_to_none = True
+
+
 # Declare compiler directives
 directive_defaults = {
     'boundscheck' : True,
@@ -81,6 +91,8 @@ directive_defaults = {
     'auto_cpdef': False,
     'cdivision': False, # was True before 0.12
     'cdivision_warnings': False,
+    'overflowcheck': False,
+    'overflowcheck.fold': True,
     'always_allow_keywords': False,
     'allow_none_for_extension_args': True,
     'wraparound' : True,
@@ -89,6 +101,9 @@ directive_defaults = {
     'final' : False,
     'internal' : False,
     'profile': False,
+    'no_gc_clear': False,
+    'linetrace': False,
+    'annotation_typing': False,  # read type declarations from Python function annotations
     'infer_types': None,
     'infer_types.verbose': False,
     'autotestdict': True,
@@ -97,6 +112,10 @@ directive_defaults = {
     'language_level': 2,
     'fast_getattr': False, # Undocumented until we come up with a better way to handle this everywhere.
     'py2_import': False, # For backward compatibility of Cython's source code in Py3 source mode
+    'c_string_type': 'bytes',
+    'c_string_encoding': '',
+    'type_version_tag': True,   # enables Py_TPFLAGS_HAVE_VERSION_TAG on extension types
+    'unraisable_tracebacks': False,
 
     # set __file__ and/or __path__ to known source/target path at import time (instead of not having them available)
     'set_initial_path' : None,  # SOURCEFILE or "/full/path/to/module"
@@ -108,9 +127,12 @@ directive_defaults = {
     'warn.unused': False,
     'warn.unused_arg': False,
     'warn.unused_result': False,
+    'warn.multiple_declarators': True,
 
 # optimizations
-    'optimize.inline_defnode_calls': False,
+    'optimize.inline_defnode_calls': True,
+    'optimize.unpack_method_calls': True,   # increases code size when True
+    'optimize.use_switch': True,
 
 # remove unreachable code
     'remove_unreachable': True,
@@ -125,7 +147,9 @@ directive_defaults = {
 
 # experimental, subject to change
     'binding': None,
-    'experimental_cpp_class_def': False
+    'freelist': 0,
+
+    'formal_grammar': False,
 }
 
 # Extra warning directives
@@ -135,6 +159,50 @@ extra_warnings = {
     'warn.unused': True,
 }
 
+def one_of(*args):
+    def validate(name, value):
+        if value not in args:
+            raise ValueError("%s directive must be one of %s, got '%s'" % (
+                name, args, value))
+        else:
+            return value
+    return validate
+
+
+def normalise_encoding_name(option_name, encoding):
+    """
+    >>> normalise_encoding_name('c_string_encoding', 'ascii')
+    'ascii'
+    >>> normalise_encoding_name('c_string_encoding', 'AsCIi')
+    'ascii'
+    >>> normalise_encoding_name('c_string_encoding', 'us-ascii')
+    'ascii'
+    >>> normalise_encoding_name('c_string_encoding', 'utF8')
+    'utf8'
+    >>> normalise_encoding_name('c_string_encoding', 'utF-8')
+    'utf8'
+    >>> normalise_encoding_name('c_string_encoding', 'deFAuLT')
+    'default'
+    >>> normalise_encoding_name('c_string_encoding', 'default')
+    'default'
+    >>> normalise_encoding_name('c_string_encoding', 'SeriousLyNoSuch--Encoding')
+    'SeriousLyNoSuch--Encoding'
+    """
+    if not encoding:
+        return ''
+    if encoding.lower() in ('default', 'ascii', 'utf8'):
+        return encoding.lower()
+    import codecs
+    try:
+        decoder = codecs.getdecoder(encoding)
+    except LookupError:
+        return encoding  # may exists at runtime ...
+    for name in ('ascii', 'utf8'):
+        if codecs.getdecoder(name) == decoder:
+            return name
+    return encoding
+
+
 # Override types possibilities above, if needed
 directive_types = {
     'final' : bool,  # final cdef classes and methods
@@ -143,10 +211,15 @@ directive_types = {
     'binding' : bool,
     'cfunc' : None, # decorators do not take directive value
     'ccall' : None,
+    'inline' : None,
+    'staticmethod' : None,
     'cclass' : None,
     'returns' : type,
     'set_initial_path': str,
-    }
+    'freelist': int,
+    'c_string_type': one_of('bytes', 'bytearray', 'str', 'unicode'),
+    'c_string_encoding': normalise_encoding_name,
+}
 
 for key, val in directive_defaults.items():
     if key not in directive_types:
@@ -155,6 +228,9 @@ for key, val in directive_defaults.items():
 directive_scopes = { # defaults to available everywhere
     # 'module', 'function', 'class', 'with statement'
     'final' : ('cclass', 'function'),
+    'inline' : ('function',),
+    'staticmethod' : ('function',),  # FIXME: analysis currently lacks more specific function scope
+    'no_gc_clear' : ('cclass',),
     'internal' : ('cclass',),
     'autotestdict' : ('module',),
     'autotestdict.all' : ('module',),
@@ -162,6 +238,13 @@ directive_scopes = { # defaults to available everywhere
     'set_initial_path' : ('module',),
     'test_assert_path_exists' : ('function', 'class', 'cclass'),
     'test_fail_if_path_exists' : ('function', 'class', 'cclass'),
+    'freelist': ('cclass',),
+    'annotation_typing': ('module',),  # FIXME: analysis currently lacks more specific function scope
+    # Avoid scope-specific to/from_py_functions for c_string.
+    'c_string_type': ('module',),
+    'c_string_encoding': ('module',),
+    'type_version_tag': ('module', 'cclass'),
+    'language_level': ('module',),
 }
 
 def parse_directive_value(name, value, relaxed_bool=False):
@@ -178,6 +261,19 @@ def parse_directive_value(name, value, relaxed_bool=False):
        ...
     ValueError: boundscheck directive must be set to True or False, got 'true'
 
+    >>> parse_directive_value('c_string_encoding', 'us-ascii')
+    'ascii'
+    >>> parse_directive_value('c_string_type', 'str')
+    'str'
+    >>> parse_directive_value('c_string_type', 'bytes')
+    'bytes'
+    >>> parse_directive_value('c_string_type', 'bytearray')
+    'bytearray'
+    >>> parse_directive_value('c_string_type', 'unicode')
+    'unicode'
+    >>> parse_directive_value('c_string_type', 'unnicode')
+    Traceback (most recent call last):
+    ValueError: c_string_type directive must be one of ('bytes', 'bytearray', 'str', 'unicode'), got 'unnicode'
     """
     type = directive_types.get(name)
     if not type: return None
@@ -200,6 +296,8 @@ def parse_directive_value(name, value, relaxed_bool=False):
                 name, orig_value))
     elif type is str:
         return str(value)
+    elif callable(type):
+        return type(name, value)
     else:
         assert False
 
@@ -226,6 +324,11 @@ def parse_directive_list(s, relaxed_bool=False, ignore_unknown=False,
     Traceback (most recent call last):
        ...
     ValueError: Unknown option: "unknown"
+    >>> warnings = parse_directive_list('warn.all=True')
+    >>> len(warnings) > 1
+    True
+    >>> sum(warnings.values()) == len(warnings)  # all true.
+    True
     """
     if current_settings is None:
         result = {}
@@ -236,10 +339,18 @@ def parse_directive_list(s, relaxed_bool=False, ignore_unknown=False,
         if not item: continue
         if not '=' in item: raise ValueError('Expected "=" in option "%s"' % item)
         name, value = [ s.strip() for s in item.strip().split('=', 1) ]
-        parsed_value = parse_directive_value(name, value, relaxed_bool=relaxed_bool)
-        if parsed_value is None:
-            if not ignore_unknown:
+        if name not in directive_defaults:
+            found = False
+            if name.endswith('.all'):
+                prefix = name[:-3]
+                for directive in directive_defaults:
+                    if directive.startswith(prefix):
+                        found = True
+                        parsed_value = parse_directive_value(directive, value, relaxed_bool=relaxed_bool)
+                        result[directive] = parsed_value
+            if not found and not ignore_unknown:
                 raise ValueError('Unknown option: "%s"' % name)
         else:
+            parsed_value = parse_directive_value(name, value, relaxed_bool=relaxed_bool)
             result[name] = parsed_value
     return result
