@@ -1,5 +1,5 @@
 # cython.* namespace for pure mode.
-__version__ = "0.21.2"
+__version__ = "0.22.1"
 
 
 # BEGIN shameless copy from Cython/minivect/minitypes.py
@@ -34,8 +34,6 @@ def index_type(base_type, item):
     a 2D strided array of doubles. The syntax is the same as for
     Cython memoryviews.
     """
-    assert isinstance(item, (tuple, slice))
-
     class InvalidTypeSpecification(Exception):
         pass
 
@@ -60,9 +58,13 @@ def index_type(base_type, item):
         return _ArrayType(base_type, len(item),
                           is_c_contig=step_idx == len(item) - 1,
                           is_f_contig=step_idx == 0)
-    else:
+    elif isinstance(item, slice):
         verify_slice(item)
         return _ArrayType(base_type, 1, is_c_contig=bool(item.step))
+    else:
+        # int[8] etc.
+        assert int(item) == item  # array size must be a plain integer
+        array(base_type, item)
 
 # END shameless copy
 
@@ -365,6 +367,15 @@ to_repr = {
 
 gs = globals()
 
+# note: cannot simply name the unicode type here as 2to3 gets in the way and replaces it by str
+try:
+    import __builtin__ as builtins
+except ImportError:  # Py3
+    import builtins
+
+gs['unicode'] = typedef(getattr(builtins, 'unicode', str), 'unicode')
+del builtins
+
 for name in int_types:
     reprname = to_repr(name, name)
     gs[name] = typedef(py_int, reprname)
