@@ -10,7 +10,7 @@ static CYTHON_INLINE PyObject* __Pyx_Generator_Yield_From(__pyx_GeneratorObject
     source_gen = PyObject_GetIter(source);
     if (unlikely(!source_gen))
         return NULL;
-    /* source_gen is now the iterator, make the first next() call */
+    // source_gen is now the iterator, make the first next() call
     retval = Py_TYPE(source_gen)->tp_iternext(source_gen);
     if (likely(retval)) {
         gen->yieldfrom = source_gen;
@@ -20,6 +20,26 @@ static CYTHON_INLINE PyObject* __Pyx_Generator_Yield_From(__pyx_GeneratorObject
     return NULL;
 }
 
+
+//////////////////// pep479.proto ////////////////////
+
+static void __Pyx_Generator_Replace_StopIteration(void); /*proto*/
+
+//////////////////// pep479 ////////////////////
+//@requires: Exceptions.c::GetException
+
+static void __Pyx_Generator_Replace_StopIteration(void) {
+    PyObject *exc, *val, *tb;
+    // Chain exceptions by moving StopIteration to exc_info before creating the RuntimeError.
+    // In Py2.x, no chaining happens, but the exception still stays visible in exc_info.
+    __Pyx_GetException(&exc, &val, &tb);
+    Py_XDECREF(exc);
+    Py_XDECREF(val);
+    Py_XDECREF(tb);
+    PyErr_SetString(PyExc_RuntimeError, "generator raised StopIteration");
+}
+
+
 //////////////////// Generator.proto ////////////////////
 #define __Pyx_Generator_USED
 #include <structmember.h>
@@ -37,12 +57,15 @@ typedef struct {
     PyObject *gi_weakreflist;
     PyObject *classobj;
     PyObject *yieldfrom;
+    PyObject *gi_name;
+    PyObject *gi_qualname;
     int resume_label;
-    char is_running;  // using T_BOOL for property below requires char value
+    // using T_BOOL for property below requires char value
+    char is_running;
 } __pyx_GeneratorObject;
 
 static __pyx_GeneratorObject *__Pyx_Generator_New(__pyx_generator_body_t body,
-                                                  PyObject *closure);
+                                                  PyObject *closure, PyObject *name, PyObject *qualname);
 static int __pyx_Generator_init(void);
 static int __Pyx_Generator_clear(PyObject* self);
 
@@ -56,7 +79,8 @@ static int __Pyx_PyGen_FetchStopIterationValue(PyObject **pvalue);
 //@requires: Exceptions.c::PyErrFetchRestore
 //@requires: Exceptions.c::SwapException
 //@requires: Exceptions.c::RaiseException
-//@requires: ObjectHandling.c::PyObjectCallMethod
+//@requires: ObjectHandling.c::PyObjectCallMethod1
+//@requires: CommonTypes.c::FetchCommonType
 
 static PyObject *__Pyx_Generator_Next(PyObject *self);
 static PyObject *__Pyx_Generator_Send(PyObject *self, PyObject *value);
@@ -78,6 +102,7 @@ static PyTypeObject *__pyx_GeneratorType = 0;
 static int __Pyx_PyGen_FetchStopIterationValue(PyObject **pvalue) {
     PyObject *et, *ev, *tb;
     PyObject *value = NULL;
+    int result;
 
     __Pyx_ErrFetch(&et, &ev, &tb);
 
@@ -97,7 +122,8 @@ static int __Pyx_PyGen_FetchStopIterationValue(PyObject **pvalue) {
 
     // most common case: plain StopIteration without or with separate argument
     if (likely(et == PyExc_StopIteration)) {
-        if (likely(!ev) || !PyObject_IsInstance(ev, PyExc_StopIteration)) {
+        int error = 0;
+        if (!ev || !(error = PyObject_IsInstance(ev, PyExc_StopIteration))) {
             // PyErr_SetObject() and friends put the value directly into ev
             if (!ev) {
                 Py_INCREF(Py_None);
@@ -108,16 +134,26 @@ static int __Pyx_PyGen_FetchStopIterationValue(PyObject **pvalue) {
             *pvalue = ev;
             return 0;
         }
+        if (unlikely(error == -1)) {
+            // error during isinstance() check
+            return -1;
+        }
     }
     // otherwise: normalise and check what that gives us
     PyErr_NormalizeException(&et, &ev, &tb);
-    if (unlikely(!PyObject_IsInstance(ev, PyExc_StopIteration))) {
+    result = PyObject_IsInstance(ev, PyExc_StopIteration);
+    if (unlikely(!result)) {
         // looks like normalisation failed - raise the new exception
         __Pyx_ErrRestore(et, ev, tb);
         return -1;
     }
     Py_XDECREF(tb);
     Py_DECREF(et);
+    if (unlikely(result == -1)) {
+        // error during isinstance() check
+        Py_DECREF(ev);
+        return -1;
+    }
 #if PY_VERSION_HEX >= 0x030300A0
     value = ((PyStopIterationObject *)ev)->value;
     Py_INCREF(value);
@@ -127,7 +163,7 @@ static int __Pyx_PyGen_FetchStopIterationValue(PyObject **pvalue) {
         PyObject* args = PyObject_GetAttr(ev, PYIDENT("args"));
         Py_DECREF(ev);
         if (likely(args)) {
-            value = PyObject_GetItem(args, 0);
+            value = PySequence_GetItem(args, 0);
             Py_DECREF(args);
         }
         if (unlikely(!value)) {
@@ -192,8 +228,8 @@ PyObject *__Pyx_Generator_SendEx(__pyx_GeneratorObject *self, PyObject *value) {
 #if CYTHON_COMPILING_IN_PYPY
         // FIXME: what to do in PyPy?
 #else
-        /* Generators always return to their most recent caller, not
-         * necessarily their creator. */
+        // Generators always return to their most recent caller, not
+        // necessarily their creator.
         if (self->exc_traceback) {
             PyThreadState *tstate = PyThreadState_GET();
             PyTracebackObject *tb = (PyTracebackObject *) self->exc_traceback;
@@ -220,9 +256,9 @@ PyObject *__Pyx_Generator_SendEx(__pyx_GeneratorObject *self, PyObject *value) {
 #if CYTHON_COMPILING_IN_PYPY
         // FIXME: what to do in PyPy?
 #else
-        /* Don't keep the reference to f_back any longer than necessary.  It
-         * may keep a chain of frames alive or it could create a reference
-         * cycle. */
+        // Don't keep the reference to f_back any longer than necessary.  It
+        // may keep a chain of frames alive or it could create a reference
+        // cycle.
         if (self->exc_traceback) {
             PyTracebackObject *tb = (PyTracebackObject *) self->exc_traceback;
             PyFrameObject *f = tb->tb_frame;
@@ -236,6 +272,15 @@ PyObject *__Pyx_Generator_SendEx(__pyx_GeneratorObject *self, PyObject *value) {
     return retval;
 }
 
+static CYTHON_INLINE
+PyObject *__Pyx_Generator_MethodReturn(PyObject *retval) {
+    if (unlikely(!retval && !PyErr_Occurred())) {
+        // method call must not terminate with NULL without setting an exception
+        PyErr_SetNone(PyExc_StopIteration);
+    }
+    return retval;
+}
+
 static CYTHON_INLINE
 PyObject *__Pyx_Generator_FinishDelegation(__pyx_GeneratorObject *gen) {
     PyObject *ret;
@@ -257,7 +302,7 @@ static PyObject *__Pyx_Generator_Next(PyObject *self) {
         PyObject *ret;
         // FIXME: does this really need an INCREF() ?
         //Py_INCREF(yf);
-        /* YieldFrom code ensures that yf is an iterator */
+        // YieldFrom code ensures that yf is an iterator
         gen->is_running = 1;
         ret = Py_TYPE(yf)->tp_iternext(yf);
         gen->is_running = 0;
@@ -271,6 +316,7 @@ static PyObject *__Pyx_Generator_Next(PyObject *self) {
 }
 
 static PyObject *__Pyx_Generator_Send(PyObject *self, PyObject *value) {
+    PyObject *retval;
     __pyx_GeneratorObject *gen = (__pyx_GeneratorObject*) self;
     PyObject *yf = gen->yieldfrom;
     if (unlikely(__Pyx_Generator_CheckRunning(gen)))
@@ -293,9 +339,11 @@ static PyObject *__Pyx_Generator_Send(PyObject *self, PyObject *value) {
         if (likely(ret)) {
             return ret;
         }
-        return __Pyx_Generator_FinishDelegation(gen);
+        retval = __Pyx_Generator_FinishDelegation(gen);
+    } else {
+        retval = __Pyx_Generator_SendEx(gen, value);
     }
-    return __Pyx_Generator_SendEx(gen, value);
+    return __Pyx_Generator_MethodReturn(retval);
 }
 
 //   This helper function is used by gen_close and gen_throw to
@@ -345,11 +393,7 @@ static PyObject *__Pyx_Generator_Close(PyObject *self) {
         Py_DECREF(yf);
     }
     if (err == 0)
-#if PY_VERSION_HEX < 0x02050000
-        PyErr_SetNone(PyExc_StopIteration);
-#else
         PyErr_SetNone(PyExc_GeneratorExit);
-#endif
     retval = __Pyx_Generator_SendEx(gen, NULL);
     if (retval) {
         Py_DECREF(retval);
@@ -360,13 +404,12 @@ static PyObject *__Pyx_Generator_Close(PyObject *self) {
     raised_exception = PyErr_Occurred();
     if (!raised_exception
         || raised_exception == PyExc_StopIteration
-#if PY_VERSION_HEX >= 0x02050000
         || raised_exception == PyExc_GeneratorExit
         || PyErr_GivenExceptionMatches(raised_exception, PyExc_GeneratorExit)
-#endif
         || PyErr_GivenExceptionMatches(raised_exception, PyExc_StopIteration))
     {
-        if (raised_exception) PyErr_Clear();      /* ignore these errors */
+        // ignore these errors
+        if (raised_exception) PyErr_Clear();
         Py_INCREF(Py_None);
         return Py_None;
     }
@@ -389,16 +432,14 @@ static PyObject *__Pyx_Generator_Throw(PyObject *self, PyObject *args) {
     if (yf) {
         PyObject *ret;
         Py_INCREF(yf);
-#if PY_VERSION_HEX >= 0x02050000
         if (PyErr_GivenExceptionMatches(typ, PyExc_GeneratorExit)) {
             int err = __Pyx_Generator_CloseIter(gen, yf);
             Py_DECREF(yf);
             __Pyx_Generator_Undelegate(gen);
             if (err < 0)
-                return __Pyx_Generator_SendEx(gen, NULL);
+                return __Pyx_Generator_MethodReturn(__Pyx_Generator_SendEx(gen, NULL));
             goto throw_here;
         }
-#endif
         gen->is_running = 1;
         if (__Pyx_Generator_CheckExact(yf)) {
             ret = __Pyx_Generator_Throw(yf, args);
@@ -423,11 +464,11 @@ static PyObject *__Pyx_Generator_Throw(PyObject *self, PyObject *args) {
         if (!ret) {
             ret = __Pyx_Generator_FinishDelegation(gen);
         }
-        return ret;
+        return __Pyx_Generator_MethodReturn(ret);
     }
 throw_here:
     __Pyx_Raise(typ, val, tb, NULL);
-    return __Pyx_Generator_SendEx(gen, NULL);
+    return __Pyx_Generator_MethodReturn(__Pyx_Generator_SendEx(gen, NULL));
 }
 
 static int __Pyx_Generator_traverse(PyObject *self, visitproc visit, void *arg) {
@@ -451,6 +492,8 @@ static int __Pyx_Generator_clear(PyObject *self) {
     Py_CLEAR(gen->exc_type);
     Py_CLEAR(gen->exc_value);
     Py_CLEAR(gen->exc_traceback);
+    Py_CLEAR(gen->gi_name);
+    Py_CLEAR(gen->gi_qualname);
     return 0;
 }
 
@@ -460,16 +503,23 @@ static void __Pyx_Generator_dealloc(PyObject *self) {
     PyObject_GC_UnTrack(gen);
     if (gen->gi_weakreflist != NULL)
         PyObject_ClearWeakRefs(self);
-    PyObject_GC_Track(self);
 
     if (gen->resume_label > 0) {
-        /* Generator is paused, so we need to close */
+        // Generator is paused, so we need to close
+        PyObject_GC_Track(self);
+#if PY_VERSION_HEX >= 0x030400a1
+        if (PyObject_CallFinalizerFromDealloc(self))
+#else
         Py_TYPE(gen)->tp_del(self);
         if (self->ob_refcnt > 0)
-            return;                     /* resurrected.  :( */
+#endif
+        {
+            // resurrected.  :(
+            return;
+        }
+        PyObject_GC_UnTrack(self);
     }
 
-    PyObject_GC_UnTrack(self);
     __Pyx_Generator_clear(self);
     PyObject_GC_Del(gen);
 }
@@ -482,11 +532,13 @@ static void __Pyx_Generator_del(PyObject *self) {
     if (gen->resume_label <= 0)
         return ;
 
-    /* Temporarily resurrect the object. */
+#if PY_VERSION_HEX < 0x030400a1
+    // Temporarily resurrect the object.
     assert(self->ob_refcnt == 0);
     self->ob_refcnt = 1;
+#endif
 
-    /* Save the current exception, if any. */
+    // Save the current exception, if any.
     __Pyx_ErrFetch(&error_type, &error_value, &error_traceback);
 
     res = __Pyx_Generator_Close(self);
@@ -496,19 +548,20 @@ static void __Pyx_Generator_del(PyObject *self) {
     else
         Py_DECREF(res);
 
-    /* Restore the saved exception. */
+    // Restore the saved exception.
     __Pyx_ErrRestore(error_type, error_value, error_traceback);
 
-    /* Undo the temporary resurrection; can't use DECREF here, it would
-     * cause a recursive call.
-     */
+#if PY_VERSION_HEX < 0x030400a1
+    // Undo the temporary resurrection; can't use DECREF here, it would
+    // cause a recursive call.
     assert(self->ob_refcnt > 0);
-    if (--self->ob_refcnt == 0)
-        return; /* this is the normal path out */
+    if (--self->ob_refcnt == 0) {
+        // this is the normal path out
+        return;
+    }
 
-    /* close() resurrected it!  Make it look like the original Py_DECREF
-     * never happened.
-     */
+    // close() resurrected it!  Make it look like the original Py_DECREF
+    // never happened.
     {
         Py_ssize_t refcnt = self->ob_refcnt;
         _Py_NewReference(self);
@@ -518,45 +571,101 @@ static void __Pyx_Generator_del(PyObject *self) {
     assert(PyType_IS_GC(self->ob_type) &&
            _Py_AS_GC(self)->gc.gc_refs != _PyGC_REFS_UNTRACKED);
 
-    /* If Py_REF_DEBUG, _Py_NewReference bumped _Py_RefTotal, so
-     * we need to undo that. */
+    // If Py_REF_DEBUG, _Py_NewReference bumped _Py_RefTotal, so
+    // we need to undo that.
     _Py_DEC_REFTOTAL;
 #endif
-    /* If Py_TRACE_REFS, _Py_NewReference re-added self to the object
-     * chain, so no more to do there.
-     * If COUNT_ALLOCS, the original decref bumped tp_frees, and
-     * _Py_NewReference bumped tp_allocs:  both of those need to be
-     * undone.
-     */
+    // If Py_TRACE_REFS, _Py_NewReference re-added self to the object
+    // chain, so no more to do there.
+    // If COUNT_ALLOCS, the original decref bumped tp_frees, and
+    // _Py_NewReference bumped tp_allocs:  both of those need to be
+    // undone.
 #ifdef COUNT_ALLOCS
     --Py_TYPE(self)->tp_frees;
     --Py_TYPE(self)->tp_allocs;
 #endif
+#endif
 }
 
-static PyMemberDef __pyx_Generator_memberlist[] = {
-    {(char *) "gi_running",
-#if PY_VERSION_HEX >= 0x02060000
-     T_BOOL,
+static PyObject *
+__Pyx_Generator_get_name(__pyx_GeneratorObject *self)
+{
+    Py_INCREF(self->gi_name);
+    return self->gi_name;
+}
+
+static int
+__Pyx_Generator_set_name(__pyx_GeneratorObject *self, PyObject *value)
+{
+    PyObject *tmp;
+
+#if PY_MAJOR_VERSION >= 3
+    if (unlikely(value == NULL || !PyUnicode_Check(value))) {
 #else
-     T_BYTE,
+    if (unlikely(value == NULL || !PyString_Check(value))) {
 #endif
-     offsetof(__pyx_GeneratorObject, is_running),
-     READONLY,
-     NULL},
+        PyErr_SetString(PyExc_TypeError,
+                        "__name__ must be set to a string object");
+        return -1;
+    }
+    tmp = self->gi_name;
+    Py_INCREF(value);
+    self->gi_name = value;
+    Py_XDECREF(tmp);
+    return 0;
+}
+
+static PyObject *
+__Pyx_Generator_get_qualname(__pyx_GeneratorObject *self)
+{
+    Py_INCREF(self->gi_qualname);
+    return self->gi_qualname;
+}
+
+static int
+__Pyx_Generator_set_qualname(__pyx_GeneratorObject *self, PyObject *value)
+{
+    PyObject *tmp;
+
+#if PY_MAJOR_VERSION >= 3
+    if (unlikely(value == NULL || !PyUnicode_Check(value))) {
+#else
+    if (unlikely(value == NULL || !PyString_Check(value))) {
+#endif
+        PyErr_SetString(PyExc_TypeError,
+                        "__qualname__ must be set to a string object");
+        return -1;
+    }
+    tmp = self->gi_qualname;
+    Py_INCREF(value);
+    self->gi_qualname = value;
+    Py_XDECREF(tmp);
+    return 0;
+}
+
+static PyGetSetDef __pyx_Generator_getsets[] = {
+    {(char *) "__name__", (getter)__Pyx_Generator_get_name, (setter)__Pyx_Generator_set_name,
+     (char*) PyDoc_STR("name of the generator"), 0},
+    {(char *) "__qualname__", (getter)__Pyx_Generator_get_qualname, (setter)__Pyx_Generator_set_qualname,
+     (char*) PyDoc_STR("qualified name of the generator"), 0},
+    {0, 0, 0, 0, 0}
+};
+
+static PyMemberDef __pyx_Generator_memberlist[] = {
+    {(char *) "gi_running", T_BOOL, offsetof(__pyx_GeneratorObject, is_running), READONLY, NULL},
     {0, 0, 0, 0, 0}
 };
 
 static PyMethodDef __pyx_Generator_methods[] = {
-    {__Pyx_NAMESTR("send"), (PyCFunction) __Pyx_Generator_Send, METH_O, 0},
-    {__Pyx_NAMESTR("throw"), (PyCFunction) __Pyx_Generator_Throw, METH_VARARGS, 0},
-    {__Pyx_NAMESTR("close"), (PyCFunction) __Pyx_Generator_Close, METH_NOARGS, 0},
+    {"send", (PyCFunction) __Pyx_Generator_Send, METH_O, 0},
+    {"throw", (PyCFunction) __Pyx_Generator_Throw, METH_VARARGS, 0},
+    {"close", (PyCFunction) __Pyx_Generator_Close, METH_NOARGS, 0},
     {0, 0, 0, 0}
 };
 
 static PyTypeObject __pyx_GeneratorType_type = {
     PyVarObject_HEAD_INIT(0, 0)
-    __Pyx_NAMESTR("generator"),         /*tp_name*/
+    "generator",                        /*tp_name*/
     sizeof(__pyx_GeneratorObject),      /*tp_basicsize*/
     0,                                  /*tp_itemsize*/
     (destructor) __Pyx_Generator_dealloc,/*tp_dealloc*/
@@ -568,7 +677,7 @@ static PyTypeObject __pyx_GeneratorType_type = {
 #else
     0,                                  /*reserved*/
 #endif
-    0,                                   /*tp_repr*/
+    0,                                  /*tp_repr*/
     0,                                  /*tp_as_number*/
     0,                                  /*tp_as_sequence*/
     0,                                  /*tp_as_mapping*/
@@ -578,17 +687,17 @@ static PyTypeObject __pyx_GeneratorType_type = {
     0,                                  /*tp_getattro*/
     0,                                  /*tp_setattro*/
     0,                                  /*tp_as_buffer*/
-    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC, /* tp_flags*/
+    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC | Py_TPFLAGS_HAVE_FINALIZE, /*tp_flags*/
     0,                                  /*tp_doc*/
     (traverseproc) __Pyx_Generator_traverse,   /*tp_traverse*/
     0,                                  /*tp_clear*/
     0,                                  /*tp_richcompare*/
-    offsetof(__pyx_GeneratorObject, gi_weakreflist), /* tp_weaklistoffse */
+    offsetof(__pyx_GeneratorObject, gi_weakreflist), /*tp_weaklistoffset*/
     0,                                  /*tp_iter*/
     (iternextfunc) __Pyx_Generator_Next, /*tp_iternext*/
     __pyx_Generator_methods,            /*tp_methods*/
     __pyx_Generator_memberlist,         /*tp_members*/
-    0,                                  /*tp_getset*/
+    __pyx_Generator_getsets,            /*tp_getset*/
     0,                                  /*tp_base*/
     0,                                  /*tp_dict*/
     0,                                  /*tp_descr_get*/
@@ -604,19 +713,21 @@ static PyTypeObject __pyx_GeneratorType_type = {
     0,                                  /*tp_cache*/
     0,                                  /*tp_subclasses*/
     0,                                  /*tp_weaklist*/
+#if PY_VERSION_HEX >= 0x030400a1
+    0,                                  /*tp_del*/
+#else
     __Pyx_Generator_del,                /*tp_del*/
-#if PY_VERSION_HEX >= 0x02060000
-    0,                                  /*tp_version_tag*/
 #endif
-#if PY_VERSION_HEX >= 0x030400a1 && defined(Py_TPFLAGS_HAVE_FINALIZE)
-    0,                                  /*tp_finalize*/
+    0,                                  /*tp_version_tag*/
+#if PY_VERSION_HEX >= 0x030400a1
+    __Pyx_Generator_del,                /*tp_finalize*/
 #endif
 };
 
 static __pyx_GeneratorObject *__Pyx_Generator_New(__pyx_generator_body_t body,
-                                                  PyObject *closure) {
+                                                  PyObject *closure, PyObject *name, PyObject *qualname) {
     __pyx_GeneratorObject *gen =
-        PyObject_GC_New(__pyx_GeneratorObject, &__pyx_GeneratorType_type);
+        PyObject_GC_New(__pyx_GeneratorObject, __pyx_GeneratorType);
 
     if (gen == NULL)
         return NULL;
@@ -632,18 +743,23 @@ static __pyx_GeneratorObject *__Pyx_Generator_New(__pyx_generator_body_t body,
     gen->exc_value = NULL;
     gen->exc_traceback = NULL;
     gen->gi_weakreflist = NULL;
+    Py_XINCREF(qualname);
+    gen->gi_qualname = qualname;
+    Py_XINCREF(name);
+    gen->gi_name = name;
 
     PyObject_GC_Track(gen);
     return gen;
 }
 
 static int __pyx_Generator_init(void) {
-    /* on Windows, C-API functions can't be used in slots statically */
+    // on Windows, C-API functions can't be used in slots statically
     __pyx_GeneratorType_type.tp_getattro = PyObject_GenericGetAttr;
     __pyx_GeneratorType_type.tp_iter = PyObject_SelfIter;
-    if (PyType_Ready(&__pyx_GeneratorType_type)) {
+
+    __pyx_GeneratorType = __Pyx_FetchCommonType(&__pyx_GeneratorType_type);
+    if (__pyx_GeneratorType == NULL) {
         return -1;
     }
-    __pyx_GeneratorType = &__pyx_GeneratorType_type;
     return 0;
 }
