@@ -170,10 +170,8 @@ static CYTHON_INLINE int __Pyx_PyUnicode_Equals(PyObject* s1, PyObject* s2, int
         Py_ssize_t length;
         int kind;
         void *data1, *data2;
-        #if CYTHON_PEP393_ENABLED
-        if (unlikely(PyUnicode_READY(s1) < 0) || unlikely(PyUnicode_READY(s2) < 0))
+        if (unlikely(__Pyx_PyUnicode_READY(s1) < 0) || unlikely(__Pyx_PyUnicode_READY(s2) < 0))
             return -1;
-        #endif
         length = __Pyx_PyUnicode_GET_LENGTH(s1);
         if (length != __Pyx_PyUnicode_GET_LENGTH(s2)) {
             goto return_ne;
@@ -190,7 +188,7 @@ static CYTHON_INLINE int __Pyx_PyUnicode_Equals(PyObject* s1, PyObject* s2, int
         } else if (length == 1) {
             goto return_eq;
         } else {
-            int result = memcmp(data1, data2, length * kind);
+            int result = memcmp(data1, data2, (size_t)(length * kind));
             #if PY_MAJOR_VERSION < 3
             Py_XDECREF(owned_ref);
             #endif
@@ -346,9 +344,7 @@ static CYTHON_INLINE Py_UCS4 __Pyx_GetItemInt_Unicode_Fast(PyObject* ustring, Py
 static CYTHON_INLINE Py_UCS4 __Pyx_GetItemInt_Unicode_Fast(PyObject* ustring, Py_ssize_t i,
                                                            int wraparound, int boundscheck) {
     Py_ssize_t length;
-#if CYTHON_PEP393_ENABLED
     if (unlikely(__Pyx_PyUnicode_READY(ustring) < 0)) return (Py_UCS4)-1;
-#endif
     if (wraparound | boundscheck) {
         length = __Pyx_PyUnicode_GET_LENGTH(ustring);
         if (wraparound & unlikely(i < 0)) i += length;
@@ -483,12 +479,8 @@ static CYTHON_INLINE PyObject* __Pyx_PyUnicode_Substring(
 static CYTHON_INLINE PyObject* __Pyx_PyUnicode_Substring(
             PyObject* text, Py_ssize_t start, Py_ssize_t stop) {
     Py_ssize_t length;
-#if CYTHON_PEP393_ENABLED
-    if (unlikely(PyUnicode_READY(text) == -1)) return NULL;
-    length = PyUnicode_GET_LENGTH(text);
-#else
-    length = PyUnicode_GET_SIZE(text);
-#endif
+    if (unlikely(__Pyx_PyUnicode_READY(text) == -1)) return NULL;
+    length = __Pyx_PyUnicode_GET_LENGTH(text);
     if (start < 0) {
         start += length;
         if (start < 0)
@@ -541,7 +533,7 @@ static int __Pyx_PyUnicode_Tailmatch(PyObject* s, PyObject* substr,
             result = PyUnicode_Tailmatch(s, PyTuple_GET_ITEM(substr, i),
                                          start, end, direction);
 #else
-            PyObject* sub = PySequence_GetItem(substr, i);
+            PyObject* sub = PySequence_ITEM(substr, i);
             if (unlikely(!sub)) return -1;
             result = PyUnicode_Tailmatch(s, sub, start, end, direction);
             Py_DECREF(sub);
@@ -567,10 +559,8 @@ static int __Pyx_PyBytes_SingleTailmatch(PyObject* self, PyObject* arg, Py_ssize
     Py_ssize_t sub_len;
     int retval;
 
-#if PY_VERSION_HEX >= 0x02060000
     Py_buffer view;
     view.obj = NULL;
-#endif
 
     if ( PyBytes_Check(arg) ) {
         sub_ptr = PyBytes_AS_STRING(arg);
@@ -583,15 +573,10 @@ static int __Pyx_PyBytes_SingleTailmatch(PyObject* self, PyObject* arg, Py_ssize
     }
 #endif
     else {
-#if PY_VERSION_HEX < 0x02060000
-        if (unlikely(PyObject_AsCharBuffer(arg, &sub_ptr, &sub_len)))
-            return -1;
-#else
         if (unlikely(PyObject_GetBuffer(self, &view, PyBUF_SIMPLE) == -1))
             return -1;
         sub_ptr = (const char*) view.buf;
         sub_len = view.len;
-#endif
     }
 
     if (end > self_len)
@@ -612,14 +597,12 @@ static int __Pyx_PyBytes_SingleTailmatch(PyObject* self, PyObject* arg, Py_ssize
     }
 
     if (start + sub_len <= end)
-        retval = !memcmp(self_ptr+start, sub_ptr, sub_len);
+        retval = !memcmp(self_ptr+start, sub_ptr, (size_t)sub_len);
     else
         retval = 0;
 
-#if PY_VERSION_HEX >= 0x02060000
     if (view.obj)
         PyBuffer_Release(&view);
-#endif
 
     return retval;
 }
@@ -635,7 +618,7 @@ static int __Pyx_PyBytes_Tailmatch(PyObject* self, PyObject* substr, Py_ssize_t
             result = __Pyx_PyBytes_SingleTailmatch(self, PyTuple_GET_ITEM(substr, i),
                                                    start, end, direction);
 #else
-            PyObject* sub = PySequence_GetItem(substr, i);
+            PyObject* sub = PySequence_ITEM(substr, i);
             if (unlikely(!sub)) return -1;
             result = __Pyx_PyBytes_SingleTailmatch(self, sub, start, end, direction);
             Py_DECREF(sub);
@@ -715,7 +698,7 @@ static CYTHON_INLINE PyObject* __Pyx_PyBytes_Join(PyObject* sep, PyObject* value
 
 #if !CYTHON_COMPILING_IN_CPYTHON
 static CYTHON_INLINE PyObject* __Pyx_PyBytes_Join(PyObject* sep, PyObject* values) {
-    return PyObject_CallMethodObjArgs(sep, PYIDENT("join"), values, NULL)
+    return PyObject_CallMethodObjArgs(sep, PYIDENT("join"), values, NULL);
 }
 #endif
 
@@ -725,6 +708,7 @@ static CYTHON_INLINE PyObject* __Pyx_PyBytes_Join(PyObject* sep, PyObject* value
 static CYTHON_INLINE int __Pyx_PyByteArray_AppendObject(PyObject* bytearray, PyObject* value);
 
 //////////////////// ByteArrayAppendObject ////////////////////
+//@requires: TypeConversion.c::PyLongInternals
 //@requires: ByteArrayAppend
 
 static CYTHON_INLINE int __Pyx_PyByteArray_AppendObject(PyObject* bytearray, PyObject* value) {
@@ -737,6 +721,17 @@ static CYTHON_INLINE int __Pyx_PyByteArray_AppendObject(PyObject* bytearray, PyO
         }
         ival = (unsigned char) (PyString_AS_STRING(value)[0]);
     } else
+#else
+#if CYTHON_USE_PYLONG_INTERNALS
+    if (likely(PyLong_CheckExact(value)) && likely(Py_SIZE(value) == 1 || Py_SIZE(value) == 0)) {
+        if (Py_SIZE(value) == 0) {
+            ival = 0;
+        } else {
+            ival = ((PyLongObject*)value)->ob_digit[0];
+            if (unlikely(ival > 255)) goto bad_range;
+        }
+    } else
+#endif
 #endif
     {
         // CPython calls PyNumber_Index() internally
@@ -744,11 +739,13 @@ static CYTHON_INLINE int __Pyx_PyByteArray_AppendObject(PyObject* bytearray, PyO
         if (unlikely((ival < 0) | (ival > 255))) {
             if (ival == -1 && PyErr_Occurred())
                 return -1;
-            PyErr_SetString(PyExc_ValueError, "byte must be in range(0, 256)");
-            return -1;
+            goto bad_range;
         }
     }
     return __Pyx_PyByteArray_Append(bytearray, ival);
+bad_range:
+    PyErr_SetString(PyExc_ValueError, "byte must be in range(0, 256)");
+    return -1;
 }
 
 //////////////////// ByteArrayAppend.proto ////////////////////
@@ -756,7 +753,7 @@ static CYTHON_INLINE int __Pyx_PyByteArray_AppendObject(PyObject* bytearray, PyO
 static CYTHON_INLINE int __Pyx_PyByteArray_Append(PyObject* bytearray, int value);
 
 //////////////////// ByteArrayAppend ////////////////////
-//@requires: ObjectHandling.c::PyObjectCallMethod
+//@requires: ObjectHandling.c::PyObjectCallMethod1
 
 static CYTHON_INLINE int __Pyx_PyByteArray_Append(PyObject* bytearray, int value) {
     PyObject *pyval, *retval;
