@@ -1486,20 +1486,18 @@ class VersionDependencyExcluder:
                 return True
         return False
 
+
 class FileListExcluder:
 
     def __init__(self, list_file, verbose=False):
         self.verbose = verbose
         self.excludes = {}
-        self._list_file = list_file
-        f = open(list_file)
-        try:
-            for line in f.readlines():
+        self._list_file = os.path.relpath(list_file)
+        with open(list_file) as f:
+            for line in f:
                 line = line.strip()
                 if line and line[0] != '#':
                     self.excludes[line.split()[0]] = True
-        finally:
-            f.close()
 
     def __call__(self, testname, tags=None):
         exclude = (testname in self.excludes
@@ -1509,6 +1507,7 @@ class FileListExcluder:
                   % (testname, self._list_file))
         return exclude
 
+
 class TagsSelector:
 
     def __init__(self, tag, value):
@@ -1932,7 +1931,7 @@ def runtests(options, cmd_args, coverage=None):
 
     try:
         import jedi
-        if list(map(int, re.findall('[0-9]+', jedi.__version__ or '0'))) < [0, 8, 1]:
+        if not ([0, 8, 1] <= list(map(int, re.findall('[0-9]+', jedi.__version__ or '0'))) < [0, 9]):
             raise ImportError
     except (ImportError, AttributeError, TypeError):
         exclude_selectors.append(RegExSelector('Jedi'))
